# Require relative to this file so we can load from a module without
# pluginsyncing
require 'pathname'
module_base = Pathname.new(__FILE__).dirname

require module_base + 'config'

require 'puppet/network/http/connection'
require 'puppet/network/http/compression'

module PuppetPE
  module Puppetlabs
    module PEConsole
      module ConsoleHTTP
        include Puppet::Network::HTTP::Compression.module

        def do_request
          connection = new_connection

          response = yield(connection, get_default_headers)

          if response.is_a? Net::HTTPSuccess
            uncompress_body(response)
          else
            message = "Error #{response.code} while communicating with #{server} on port #{port}:\n" +
                      "#{(response.body||'').empty? ? response.message : uncompress_body(response)}"
            raise Puppet::Error, message
          end
        end

        private

        ## Connection helpers ##

        # Creates a new connection using our certname validator
        def new_connection
          Puppet::Network::HttpPool.http_ssl_instance(server, port)
        end

        # Provide appropriate headers.
        def get_default_headers
          add_accept_encoding({
            'X-Proxy-User' => 'system',
            'Accept'       => 'text/yaml',
          })
        end


        ## Configuration helpers ##

        def load_config
          @console_config = PuppetPE::Puppetlabs::PEConsole::Config.load
        end

        def config
          @console_config
        end

        def server
          @console_config.server
        end

        def certificate_name
          @console_config.certificate_name
        end

        def port
          @console_config.port
        end

      end
    end
  end
end

