require 'spec_helper'
require 'puppet/network/http_pool'
require 'puppetpe/puppetlabs/pe_console/console_http'

class TestClassUsingConsoleHTTP
  include PuppetPE::Puppetlabs::PEConsole::ConsoleHTTP

  def initialize
    super
    load_config
  end
end

describe "The console_http mixin" do
  subject { TestClassUsingConsoleHTTP.new }

  let(:https) { stub_everything "https" }

  before(:each) do
    subject.expects(:server).at_least(1).returns('testing')
    subject.expects(:port).at_least(1).returns(8080)
    subject.expects(:uncompress_body).once.returns(nil)

    # The most important expectation -- this is really what we're testing!
    Puppet::Network::HttpPool.expects(:http_ssl_instance).with(
      'testing', 8080
    ).returns https
  end

  describe "when making a request" do
    Net::HTTPResponse::CODE_TO_OBJ.each do |code, klass|
      if code.to_i >= 200 and code.to_i < 300
        it "should succeed on http code #{code}" do
          response = klass.new('1.1', code, '')
          captured_connection = nil

          Puppet.expects(:err).never

          subject.do_request do |connection, headers|
            captured_connection = connection
            response
          end

          captured_connection.should == https
        end
      end

      if code.to_i >= 300 && ![301, 302, 307].include?(code.to_i)
        it "should raise an error on http code #{code}" do
          response = klass.new('1.1', code, '')
          response.expects(:body).returns(stub_everything "body")
          captured_connection = nil

          expect {
          subject.do_request do |connection, headers|
            captured_connection = connection
            response
          end
          }.to raise_error


          captured_connection.should == https
        end
      end
    end
  end
end
